/*
#############################################################
## The original file is from
## http://cran.r-project.org/src/base/R-2/R-2.11.1.tar.gz
## ORIGINAL Copyright #########################################
#  File src/library/stats/R/quantile.R
#  Part of the R package, http://www.R-project.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#############################################################

%SQuantile() macro is a SAS port of the quantile() function
that is a part of R stat package.

Ported by Kengo NAGASHIMA. (06/09/2010)
*/

/********************************************************
  Macro name: %SQuantile()
  Author: Kengo NAGASHIMA
  Version: 1.0.0
  Last Updated Date: 06/09/2010
  System Requirement: Base SAS
********************************************************/
/********************************************************
  Sample quantile function
    data: input dataset name
    var: variable name whose sample quantile is wanted
    prob: quantile point [0, 1]
    out: output dataset name
    method: Nine sample quantile calculation method are
      implemented.  see R documentation file. default is 8.
********************************************************/

%macro SQuantile(data, var, prob, out, method = 8);
data _null_;
  set &data.;
  prob = min(1, max(0, &prob.));
  p = compress(translate(cat(prob*100), "_", "."));
  call symput("n", _n_); call symput("p", p);
proc rank data = &data. out = &out.;
  var &var.; ranks &var._r;
proc sort data = &out.; by &var._r;
%if "&method." = "7" %then %do;
data _null_;
  prob = min(1, max(0, &prob.));
  index = 1 + (&n. - 1) * prob;
  lo = floor(index);
  hi = ceil(index);
  h = index - lo;
  call symput("lo", lo); call symput("hi", hi); call symput("h", h);
data &out.;
  merge
    &out.(firstobs=&lo. obs=&lo. rename=(&var.=&var._l))
    &out.(firstobs=&hi. obs=&hi. rename=(&var.=&var._h));
  q&p. = (1 - &h.) * &var._l + &h. * &var._h;
  keep q&p.;
run;
%end;
%else %do;
data _null_;
  prob = min(1, max(0, &prob.));
  %if "&method." = "1" %then %do;
    nppm = &n. * prob; j = floor(nppm);
    if (nppm > j) then h = 1; else h = 0;
  %end;
  %else %if "&method." = "2" %then %do;
    nppm = &n. * prob; j = floor(nppm);
    if (nppm > j) then h = 1; else h = 0.5;
  %end;
  %else %if "&method." = "3" %then %do;
    nppm = &n. * prob - 0.5; j = floor(nppm);
    if (nppm = j  & mod(j, 2) = 0) then h = 0; else h = 1;
  %end;
  %else %do;
    %if "&method." = "4" %then %do; a = 0; b = 1; %end;
    %if "&method." = "5" %then %do; a = 0.5; b = 0.5; %end;
    %if "&method." = "6" %then %do; a = 0; b = 0; %end;
    %if "&method." = "8" %then %do; a = 1/3; b = 1/3; %end;
    %if "&method." = "9" %then %do; a = 3/8; b = 3/8; %end;
    fuzz = 4 * constant('MACEPS');
    nppm = a + prob * (&n. + 1 - a - b);
    j = floor(nppm + fuzz);
    h = nppm - j;
    if abs(h) < fuzz then h = 0;
    else h = h;
  %end;
  call symput("j2", j+2); call symput("j3", j+3);
  call symput("h", put(h, 32.16));
data &out.;
  set
    &out.(firstobs=1 obs=1) &out.(firstobs=1 obs=1) &out.
    &out.(firstobs=&n. obs=&n.) &out.(firstobs=&n. obs=&n.);
data &out.;
  merge
    &out.(firstobs=&j2. obs=&j2. rename=(&var.=&var._l))
    &out.(firstobs=&j3. obs=&j3. rename=(&var.=&var._h));
  if &h. = 0 then q&p. = &var._l;
  else if &h. = 1 then q&p. = &var._h;
  else if (&h. > 0 & &h. < 1) then
    q&p. = (1 - &h.) * &var._l + &h. * &var._h;
  put &var._l;
  put &var._h;
  keep q&p.;
%end;
%mend SQuantile;
